<?php

    /**
     * Modul für "Bezahlen mit Amazon"
     * @author Daschmi (12.03.2016)
     */
    class wpsg_mod_amazon_v2 extends wpsg_mod_basic {

        var $lizenz = 1;
        var $id = 3141;
        var $hilfeURL = 'http://wpshopgermany.de/?p=3462587';
        var $version = "9.9.9";
        var $free = false;

        public function __construct() {

            parent::__construct();

            $this->name = __('AmazonPay V2', 'wpsg');
            $this->group = __('Zahlungsarten', 'wpsg');
            $this->desc = __('Ermöglicht die Anmeldung / Zahlung mit Amazon.', 'wpsg');

        } // public function __construct()

        public function install() {

            $this->shop->checkDefault('wpsg_mod_amazon_v2_aktiv', '1');
            $this->shop->checkDefault('wpsg_mod_amazon_v2_name', __('AmazonPay', 'wpsg'), false, true);
            $this->shop->checkDefault('wpsg_mod_amazon_v2_hint', __('Bezahlen Sie ihre Bestellung einfach mit ihrem Amazon Konto.', 'wpsg'), false, true);
            $this->shop->checkDefault('wpsg_mod_amazon_v2_subject', __('O%order_id% - K%kunde_id%', 'wpsg'), false, true);

        } // public function install()

        public function settings_edit() {

            $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_amazon_v2/settings_edit.phtml');

        } // public function settings_edit()

        public function settings_save() {

            $this->shop->update_option('wpsg_mod_amazon_v2_seller_id', $_REQUEST['wpsg_mod_amazon_v2_seller_id'], false, false, WPSG_SANITIZE_APIKEY);
            $this->shop->update_option('wpsg_mod_amazon_v2_store_id', $_REQUEST['wpsg_mod_amazon_v2_store_id'], false, false, WPSG_SANITIZE_APIKEY);
            $this->shop->update_option('wpsg_mod_amazon_v2_public_key_id', $_REQUEST['wpsg_mod_amazon_v2_public_key_id'], false, false, WPSG_SANITIZE_TEXTFIELD);

            if (isset($_FILES['wpsg_mod_amazon_v2_private_key']) && file_exists(($_FILES['wpsg_mod_amazon_v2_private_key']['tmp_name']))) {

                $b = move_uploaded_file($_FILES['wpsg_mod_amazon_v2_private_key']['tmp_name'], $this->getKey(true).'/key.pem');

                if ($b) $this->shop->addBackendMessage(__('Upload der Schlüsseldatei war erfolgreich.', 'wpsg'));
                else $this->shop->addBackendError(__('Beim Upload der Schlüsseldatei gab es ein Problem.', 'wpsg'));

            }

            $this->shop->update_option('wpsg_mod_amazon_v2_sandbox', $_REQUEST['wpsg_mod_amazon_v2_sandbox'], false, false, WPSG_SANITIZE_CHECKBOX);

            $this->shop->update_option('wpsg_mod_amazon_v2_aktiv', $_REQUEST['wpsg_mod_amazon_v2_aktiv'], false, false, WPSG_SANITIZE_CHECKBOX);
            $this->shop->update_option('wpsg_mod_amazon_v2_name', $_REQUEST['wpsg_mod_amazon_v2_name'], false, true, WPSG_SANITIZE_TEXTFIELD);
            $this->shop->update_option('wpsg_mod_amazon_v2_subject', $_REQUEST['wpsg_mod_amazon_v2_subject'], false, true, WPSG_SANITIZE_TEXTFIELD);
            $this->shop->update_option('wpsg_mod_amazon_v2_hint', $_REQUEST['wpsg_mod_amazon_v2_hint'], false, true, WPSG_SANITIZE_TEXTFIELD);

            $this->shop->update_option('wpsg_mod_amazon_v2_gebuehr', $_REQUEST['wpsg_mod_amazon_v2_gebuehr'], false, false, WPSG_SANITIZE_FLOAT);
            $this->shop->update_option('wpsg_mod_amazon_v2_mwst', $_REQUEST['wpsg_mod_amazon_v2_mwst'], false, false, WPSG_SANITIZE_TAXKEY);
            $this->shop->update_option('wpsg_mod_amazon_v2_mwstland', $_REQUEST['wpsg_mod_amazon_v2_mwstland'], false, false, WPSG_SANITIZE_CHECKBOX);

        } // public function settings_save()

        public function basket_checkoutAction(&$basketController) {

            if (wpsg_isSizedString($_REQUEST['amazon']['OrderReference'])) {

                $_SESSION['wpsg_mod_amazon_v2_order_reference'] = $_REQUEST['amazon']['OrderReference'];

            }

        }

        public function basket_inner_prebutton(&$basket_view) {

            $view = $this->getPaymentButton($this->shop->getUrl(wpsg_ShopController::URL_BASKET, 'wpsg_mod_amazon_v2', 'checkoutReviewReturnUrl', [], true));

            $this->shop->view['wpsg_mod_amazon'] = $view;
            $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_amazon_v2/basket_inner_prebutton.phtml');

        }

        public function addPayment(&$arPayment) {

            if (is_admin() || $this->shop->get_option('wpsg_mod_amazon_v2_aktiv') == '1') {

                $arPayment[$this->id] = array(
                    'id' => $this->id,
                    'name' => __($this->shop->get_option('wpsg_mod_amazon_v2_name'), 'wpsg'),
                    'price' => $this->shop->get_option('wpsg_mod_amazon_v2_gebuehr'),
                    'tax_key' => $this->shop->get_option('wpsg_mod_amazon_v2_mwst'),
                    'mwst_null' => $this->shop->get_option('wpsg_mod_amazon_v2_mwstland'),
                    'hint' => __($this->shop->get_option('wpsg_mod_amazon_v2_hint'), 'wpsg'),
                    'logo' => $this->shop->getRessourceURL('mods/mod_amazon_v2/gfx/logo_100x25.png')
                );

            }

        }

        public function order_done(&$order_id, &$done_view) {

            // Bestellungen mit 0 geben nix aus
            if ($done_view['basket']['sum']['preis_gesamt_brutto'] <= 0) return;

            $oOrder = wpsg_order::getInstance($order_id);

            // Bestellungen die nicht mit Amazon beglichen werden sollen ignorieren
            if ($oOrder->getPaymentID() != $this->id) return;

            $view['order_id'] = $order_id;

            if (isset($_SESSION['wpsg_mod_amazon_v2_order_reference'])) {

                $oOrder->setMeta('mod_amazon_v2_order_reference', $_SESSION['wpsg_mod_amazon_v2_order_reference']);
                unset($_SESSION['wpsg_mod_amazon_v2_order_reference']);

            }

            if (isset($_SESSION['wpsg_mod_amazon_v2_amazonCheckoutSessionId'])) {

                $oOrder->setMeta('mod_amazon_v2_session', $_SESSION['wpsg_mod_amazon_v2_amazonCheckoutSessionId']);
                unset($_SESSION['wpsg_mod_amazon_v2_amazonCheckoutSessionId']);

            }

            if ($oOrder->getMeta('mod_amazon_v2_session') !== '') {

                $res = $this->getPaymentInfo($order_id, true);

                $view['state'] = $res['statusDetails']['state'];

                if ($view['state'] === 'Completed') {

                } else if ($view['state'] === 'Open') {

                    $res = $this->updatePayment($order_id);

                    $view['session_id'] = $oOrder->getMeta('mod_amazon_v2_session');
                    $view['payURL'] = $res['webCheckoutDetails']['amazonPayRedirectUrl'];

                } else {

                    //$res = $this->updatePayment($order_id);
                    //echo '<a href="'.$res['webCheckoutDetails']['amazonPayRedirectUrl'].'">PAY</a>';

                    //$res = $this->completePayment($order_id);

                    die(wpsg_debug($res));

                }

            } else {

                $view['state'] = 'Unpayed';

                $view += $this->getPaymentButton(
                    $this->shop->getDoneURL($oOrder->getId(), 'wpsg_mod_amazon_v2', 'checkoutOrderDone')
                );

            }

            $this->shop->view['wpsg_mod_amazon_v2'] = $view;
            $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_amazon_v2/order_done.phtml');

        }

        public function order_view_afterpayment(&$order_id) {

            $oOrder = wpsg_order::getInstance($order_id);

            if ($oOrder->getPaymentID() != $this->id) return;

            $view = [
                'chargeId' => $oOrder->getMeta('mod_amazon_v2_chargeId'),
                'refundId' => $oOrder->getMeta('mod_amazon_v2_refundId'),
                'refundState' => $oOrder->getMeta('mod_amazon_v2_refundState'),
                'sesion_id' => $oOrder->getMeta('mod_amazon_v2_session'),
                'state' => $oOrder->getMeta('mod_amazon_v2_state'),
                'referenceID' => $oOrder->getMeta('mod_amazon_v2_AmazonOrderReferenceId')
            ];

            $this->shop->view['wpsg_mod_amazon_v2'] = $view;

            $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_amazon_v2/order_view_afterpayment.phtml');

        }

        public function order_ajax() {

            $order_id = intval($_REQUEST['edit_id']);
            $do = $_REQUEST['do'];

            if (wpsg_isSizedString($do, 'update')) {

                $this->getPaymentInfo($order_id);

                $this->shop->addBackendMessage(__('Status abgefragt.', 'wpsg'));;

            } else if (wpsg_isSizedString($do, 'updateRefund')) {

                try {

                    $this->getRefundInfo($order_id);
                    $this->shop->addBackendMessage(__('Erstattungsstatus abgefragt.', 'wpsg'));

                } catch (\Exception $e) {

                    $this->shop->addBackendError(
                        wpsg_translate(__('Es ist ein Fehler aufgetreten: #1#', 'wpsg'), $e->getMessage())
                    );

                }

            } else if (wpsg_isSizedString($do, 'refund')) {

                $oOrder = wpsg_order::getInstance($order_id);

                if ($oOrder->getMeta('mod_amazon_v2_chargeId') !== '') {

                    $this->refundPayment($order_id);

                    $this->shop->addBackendMessage(__('Rückzahlung veranlasst.', 'wpsg'));;

                } else {

                    $this->shop->addBackendError(__('Rückzahlung nicht möglich.', 'wpsg'));

                }

            }

            $this->shop->redirect(wpsg_admin_url('Order', 'view', ['edit_id' => $order_id]));

        }

        /* Redirect */

        /**
         * Wird für die Variante mit den Iframes aufgerufen nach der Bestellung
         * https://developer.amazon.com/de/docs/eu/amazon-pay-onetime/sca-upgrade.html?ld=APNALPADirect
         * Schritt 2ff
         */
        public function placeOrderRedirect() {

            $order_id = intval($_REQUEST['order_id']);
            $oOrder = wpsg_order::getInstance($order_id);

            $client = $this->getOriginClientApi();

            // Schritt 2
            $requestParameters = [];
            $requestParameters['amazon_order_reference_id'] = $oOrder->getMeta('mod_amazon_v2_order_reference');
            $requestParameters['amount'] = $oOrder->getToPay(WPSG_BRUTTO);
            $requestParameters['currency_code'] = 'EUR';
            $requestParameters['seller_note'] = 'Thank you for your order.';
            $requestParameters['store_name'] = 'Name of your shop';
            $requestParameters['seller_order_id'] = $oOrder->getNr();

            $res = $client->SetOrderAttributes($requestParameters)->toArray();
            $oOrder->addLogEntry('AmazonPay SetOrderAttributes', print_r($res, true));

            if (isset($res['SetOrderAttributesResult']['OrderReferenceDetails']['AmazonOrderReferenceId'])) $oOrder->setMeta('mod_amazon_v2_AmazonOrderReferenceId', $res['SetOrderAttributesResult']['OrderReferenceDetails']['AmazonOrderReferenceId']);

            // Schritt 3
            $requestParameters = [];
            $requestParameters['amazon_order_reference_id'] = $oOrder->getMeta('mod_amazon_v2_order_reference');
            $requestParameters['success_url'] = $this->shop->getDoneURL($oOrder->getId(), 'wpsg_mod_amazon_v2', 'placeOrderSuccess');
            $requestParameters['failure_url'] = $this->shop->getDoneURL($oOrder->getId(), 'wpsg_mod_amazon_v2', 'placeOrderFailure');

            $res = $client->confirmOrderReference($requestParameters)->toArray();
            $oOrder->addLogEntry('AmazonPay confirmOrderReference', print_r($res, true));

        }

        public function placeOrderSuccessRedirect() {

            $order_id = intval($_REQUEST['order_id']);
            $oOrder = wpsg_order::getInstance($order_id);

            $client = $this->getOriginClientApi();

            // Authorize
            $res = $client->authorize([
                'amazon_order_reference_id' => $oOrder->getMeta('mod_amazon_v2_order_reference'),
                'authorization_amount' => $oOrder->getToPay(),
                'authorization_reference_id' => uniqid('Authorization-'.$oOrder->getId()),
                'soft_descriptor' => $this->shop->replaceUniversalPlatzhalter(__($this->shop->get_option('wpsg_mod_amazon_subject'), 'wpsg'), $order_id),
                'seller_authorization_note' => $this->shop->replaceUniversalPlatzhalter(__($this->shop->get_option('wpsg_mod_amazon_subject'), 'wpsg'), $order_id),
                'currency_code' => 'EUR',
                'capture_now' => true
            ])->toArray();

            if (isset($res['Error'])) {

                $oOrder->addLogEntry('AmazonPay Error', print_r($res, true));

                throw new \Exception($res['Error']['Message']);

            } else $oOrder->addLogEntry('AmazonPay Authorize', print_r($res, true));

            if (isset($res['AuthorizeResult']['AuthorizationDetails']['IdList']['member'])) $oOrder->setMeta('mod_amazon_v2_chargeId', $res['AuthorizeResult']['AuthorizationDetails']['IdList']['member']);

            $AmazonAuthorizationId = $res['AuthorizeResult']['AuthorizationDetails']['AmazonAuthorizationId'];
            $oOrder->setMeta('wpsg_mod_amazon_v2_amazon_authorization_id', $AmazonAuthorizationId);

            // Capture
            /*
            $res = $client->capture([
                'amazon_authorization_id' => $AmazonAuthorizationId,
                'capture_amount' => $oOrder->getToPay(),
                'currency_code' => 'EUR',
                'capture_reference_id' => uniqid('Capture-'.$oOrder->getId())
            ]);

            wpsg_debug($res->toArray());
            */

            $res = $client->closeOrderReference(array(
                'amazon_order_reference_id' => $oOrder->getMeta('mod_amazon_v2_order_reference'),
                'closure_reason' => ''
            ));

            $oOrder->addLogEntry('AmazonPay closeOrder', print_r($res, true));

            $this->shop->redirect($this->shop->getDoneURL($oOrder->getId()));

        }

        public function placeOrderFailureRedirect() {

            die(wpsg_debug($_REQUEST));

        }

        public function loginRedirect() {

            $_SESSION['wpsg']['checkout']['payment'] = $this->id;
            $_SESSION['wpsg_mod_amazon_v2_access_token'] = $_REQUEST['access_token'];

            $this->shop->redirect($this->shop->getURL(wpsg_ShopController::URL_BASKET));

        }

        public function checkoutReviewReturnUrlRedirect() {

            $client = $this->getClientApi();
            $result = $client->getCheckoutSession($_REQUEST['amazonCheckoutSessionId']);
            $res = json_decode($result['response'], true);

            if (is_array($res)) {

                $_SESSION['wpsg_mod_amazon_v2_amazonCheckoutSessionId'] = $_REQUEST['amazonCheckoutSessionId'];

                // Zahlungsart
                $_SESSION['wpsg']['checkout']['payment'] = $this->id;

                // EMail
                if (wpsg_isSizedString($res['buyer']['email'])) {

                    $_SESSION['wpsg']['checkout']['email'] = $res['buyer']['email'];
                    $_SESSION['wpsg']['checkout']['email2'] = $res['buyer']['email'];

                }

                // Rechnungsadresse
                list($vname, $name) = wpsg_explodeName($res['billingAddress']['name']);

                if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['vname'])) $_SESSION['wpsg']['checkout']['vname'] = $vname;
                if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['name'])) $_SESSION['wpsg']['checkout']['name'] = $name;

                for ($i = 1; $i <= 3; $i ++) {

                    if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['strasse']) && wpsg_isSizedString($res['billingAddress']['addressLine'.$i])) $_SESSION['wpsg']['checkout']['strasse'] = $res['billingAddress']['addressLine'.$i];

                }

                if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['plz'])) $_SESSION['wpsg']['checkout']['plz'] = $res['billingAddress']['postalCode'];
                if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['ort'])) $_SESSION['wpsg']['checkout']['ort'] = $res['billingAddress']['city'];
                if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['tel'])) $_SESSION['wpsg']['checkout']['tel'] = $res['billingAddress']['phoneNumber'];

                // Land wenn gefunden
                $country_id = wpsg_country::getCountryIDFromCode($res['billingAddress']['countryCode']);
                if (!wpsg_isSizedInt($_SESSION['wpsg']['checkout']['land']) && wpsg_isSizedString($country_id)) $_SESSION['wpsg']['checkout']['land'] = $country_id;

                // Lieferadresse
                if (wpsg_isSizedArray($res['shippingAddress'])) {

                    if ($this->shop->hasMod('wpsg_mod_shippingadress')) {

                        list($vname, $name) = wpsg_explodeName($res['shippingAddress']['name']);

                        if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['shipping_vname'])) $_SESSION['wpsg']['checkout']['shipping_vname'] = $vname;
                        if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['shipping_name'])) $_SESSION['wpsg']['checkout']['shipping_name'] = $name;

                        for ($i = 1; $i <= 3; $i ++) {

                            if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['shipping_strasse']) && wpsg_isSizedString($res['billingAddress']['addressLine'.$i])) $_SESSION['wpsg']['checkout']['shipping_strasse'] = $res['shippingAddress']['addressLine'.$i];

                        }

                        if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['shipping_plz'])) $_SESSION['wpsg']['checkout']['shipping_plz'] = $res['shippingAddress']['postalCode'];
                        if (!wpsg_isSizedString($_SESSION['wpsg']['checkout']['shipping_ort'])) $_SESSION['wpsg']['checkout']['shipping_ort'] = $res['shippingAddress']['city'];

                        // Land wenn gefunden
                        $country_id = wpsg_country::getCountryIDFromCode($res['shippingAddress']['countryCode']);
                        if (!wpsg_isSizedInt($_SESSION['wpsg']['checkout']['shipping_land']) && wpsg_isSizedString($country_id)) $_SESSION['wpsg']['checkout']['shipping_land'] = $country_id;

                    }

                }

            }

            $this->shop->redirect($this->shop->getURL(wpsg_ShopController::URL_OVERVIEW));

        }

        public function checkoutResultReturnUrlRedirect()  {

            $oOrder = wpsg_order::getInstance(intval($_REQUEST['order_id']));

            $res = $this->completePayment($oOrder->getId());

            if ($res['statusDetails']['state'] !== 'Completed') {

                $this->shop->addFrontendError(__('Es gab Probleme mit der Zahlung über Amazon Pay.', 'wpsg'));

            }

            $this->shop->redirect($this->shop->getDoneURL($oOrder->getId()));

        }

        /**
         * Wenn die Zahlung nach der Bestellung ausgelöst wurde
         * @throws Exception
         */
        public function checkoutOrderDoneRedirect() {

            $oOrder = wpsg_order::getInstance(intval($_REQUEST['order_id']));

            $client = $this->getClientApi();
            $result = $client->getCheckoutSession($_REQUEST['amazonCheckoutSessionId']);
            $res = json_decode($result['response'], true);

            if (is_array($res)) {

                $oOrder->setMeta('mod_amazon_v2_session', $_REQUEST['amazonCheckoutSessionId']);
                $res = $this->updatePayment($oOrder->getId());

                $this->shop->redirect($res['webCheckoutDetails']['amazonPayRedirectUrl']);

            }

        }

        /*
        public function handle_order() {

            $client = $this->getClientApi();

            $res = $client->getCheckoutSession($_SESSION['wpsg_mod_amazon_v2_amazonCheckoutSessionId']);
            die(wpsg_debug(json_decode($res['response'], true)));

            $this->shop->render(WPSG_PATH_VIEW.'/mods/mod_amazon_v2/overview_button.phtml');

            return -2;

        } */

        /* */

        // Modulfunktionen

        private function getPaymentButton($url) {

            $payload = json_encode([
                'webCheckoutDetails' => [
                    'checkoutReviewReturnUrl' => $url
                ],
                'storeId' => $this->shop->get_option('wpsg_mod_amazon_v2_store_id'),
                'deliverySpecifications' => [

                ]
            ]);

            $client = $this->getClientApi();
            $signature = $client->generateButtonSignature($payload);

            return [
                'payload' => $payload,
                'signature' => $signature
            ];

        }

        private function getPaymentInfo($order_id, $noLog = false) {

            $oOrder = wpsg_order::getInstance($order_id);
            $client = $this->getClientApi();

            $session_id = $oOrder->getMeta('mod_amazon_v2_session');

            if (!wpsg_isSizedString($session_id)) throw new \Exception(__('Keine Amaonz Payments Session ID vorhanden.', 'wpsg'));

            $res = $client->getCheckoutSession($oOrder->getMeta('mod_amazon_v2_session'))['response'];
            $res = json_decode($res, true);

            if (isset($res['reasonCode'])) {

                $oOrder->addLogEntry(__('AmazonPay Error', 'wpsg'), print_r($res, true));

                throw new \Exception($res['reasonCode']);

            } else {

                if (!$noLog) $oOrder->addLogEntry(__('AmazonPay PaymentInfo', 'wpsg'), print_r($res, true));

                if (isset($res['statusDetails']['state'])) $oOrder->setMeta('mod_amazon_v2_state', $res['statusDetails']['state']);

                return $res;

            }

        }

        /**
         * @param int $order_id
         * @throws Exception
         * @return Array
         */
        private function getRefundInfo($order_id) {

            $oOrder = wpsg_order::getInstance($order_id);
            $client = $this->getClientApi();

            $refund_id = $oOrder->getMeta('mod_amazon_v2_refundId');

            if (!wpsg_isSizedString($refund_id)) throw new \Exception(__('Keine RefundID für Amazon Rückbuchung gesetzt.', 'wpsg'));

            $res = $client->getRefund($refund_id);
            $res = json_decode($res['response'], true);

            if (isset($res['reasonCode'])) {

                $oOrder->addLogEntry(__('AmazonPay Error', 'wpsg'), print_r($res, true));

                throw new \Exception($res['reasonCode']);

            } else {

                if (isset($res['statusDetails']['state'])) $oOrder->setMeta('mod_amazon_v2_refundState', $res['statusDetails']['state']);

                $oOrder->addLogEntry(__('AmazonPay RefundInfo', 'wpsg'), print_r($res, true));

                return $res;

            }

        }

        private function completePayment($order_id) {

            $oOrder = wpsg_order::getInstance($order_id);

            $payload = json_encode([
                'chargeAmount' => [
                    'amount' => $oOrder->getToPay(),
                    'currencyCode' => 'EUR'
                ]
            ]);

            $client = $this->getClientApi();

            $res = json_decode($client->completeCheckoutSession($oOrder->getMeta('mod_amazon_v2_session'), $payload)['response'], true);

            if (isset($res['statusDetails']['state'])) $oOrder->setMeta('mod_amazon_v2_state', $res['statusDetails']['state']);
            if (isset($res['chargeId'])) $oOrder->setMeta('mod_amazon_v2_chargeId', $res['chargeId']);
            if (isset($res['chargePermissionId'])) $oOrder->setMeta('mod_amazon_v2_AmazonOrderReferenceId', $res['chargePermissionId']);

            $oOrder->addLogEntry(__('AmazonPay CompletePayment', 'wpsg'), print_r($res, true));

            if ($res['statusDetails']['state'] === 'Completed') {

                if ($oOrder->__get('ext_payed')  !== '1') {

                    $this->shop->setOrderStatus($oOrder->id, 100, true);
                    $this->db->UpdateQuery(WPSG_TBL_ORDER, ['ext_payed' => '1'], " `id` = '".wpsg_q($oOrder->getId())."' ");

                }

            }

            return $res;

        }

        private function updatePayment($order_id) {

            $oOrder = wpsg_order::getInstance($order_id);

            $payload = json_encode([
                'webCheckoutDetails' => [
                    'checkoutResultReturnUrl' => $this->shop->getDoneURL($oOrder->getId(), 'wpsg_mod_amazon_v2', 'checkoutResultReturnUrl')
                ],
                'paymentDetails' => [
                    'paymentIntent' => 'AuthorizeWithCapture',
                    'canHandlePendingAuthorization' => false,
                    'softDescriptor' => 'Descriptor',
                    'chargeAmount' => [
                        'amount' => $oOrder->getToPay(),
                        'currencyCode' => 'EUR'
                    ]
                ],
                'merchantMetadata' => [
                    'merchantReferenceId' => $oOrder->getNr(),
                    'merchantStoreName' => $this->shop->get_option('wpsg_shopdata_name')
                ]
            ]);

            $client = $this->getClientApi();

            $res = json_decode($client->updateCheckoutSession($oOrder->getMeta('mod_amazon_v2_session'), $payload)['response'], true);

            if (isset($res['statusDetails']['state'])) $oOrder->setMeta('mod_amazon_v2_state', $res['statusDetails']['state']);

            $oOrder->addLogEntry(__('AmazonPay UpdatePayment', 'wpsg'), print_r($res, true));

            return $res;

        }

        private function refundPayment($order_id) {

            $oOrder = wpsg_order::getInstance($order_id);
            $refund_key = $oOrder->getMeta('mod_amazon_v2_refund_key');

            if ($refund_key === '') {

                $refund_key = wpsg_genCode(32);
                $oOrder->setMeta('mod_amazon_v2_refund_key', $refund_key);

            }

            $chargeID = $oOrder->getMeta('mod_amazon_v2_chargeId');

            $payload = json_encode([
                'chargeId' => $chargeID,
                'refundAmount' => [
                    'amount' => $oOrder->getToPay(WPSG_BRUTTO),
                    'currencyCode' => 'EUR'
                ],
                'softDescriptor' => substr('Refund '.$oOrder->getNr(), 0, 16)
            ]);

            $client = $this->getClientApi();

            $res = $client->createRefund($payload, ['x-amz-pay-idempotency-key' => $refund_key]);
            $res = json_decode($res['response'], true);

            if (isset($res['statusDetails']['state'])) $oOrder->setMeta('mod_amazon_v2_refundState', $res['statusDetails']['state']);
            if (isset($res['refundId'])) $oOrder->setMeta('mod_amazon_v2_refundId', $res['refundId']);

            $oOrder->addLogEntry(__('AmazonPay RefundPayment', 'wpsg'), print_r($res, true));

            return $res;

        }

        /**
         * @return \Amazon\Pay\API\Client
         * @throws Exception
         */
        private function getClientApi() {

            require_once __DIR__.'/mod_amazon_v2/vendor/autoload.php';

            $amazonpay_config = [
                'public_key_id' => $this->shop->get_option('wpsg_mod_amazon_v2_public_key_id'),
                'private_key' => $this->getKey(),
                'region' => 'EU',
                'sandbox' => $this->shop->get_option('wpsg_mod_amazon_v2_sandbox') === '1'
            ];

            return new Amazon\Pay\API\Client($amazonpay_config);

        }

        private function getKey($path_only = false) {

            if ($this->shop->isMultiBlog()) {

                $path = WP_CONTENT_DIR.'/'.WPSG_MB_UPLOADS.'/wpsg/wpsg_mod_amazon/';

            } else {

                $path = WP_CONTENT_DIR.'/uploads/wpsg/wpsg_mod_amazon/';

            }

            $this->shop->protectDirectory($path);

            if ($path_only) return $path;

            $path .= 'key.pem';

            if (!file_exists($path)) throw new \Exception(__('Amazon Public Key existiert nicht.'));

            return $path;

        }

    }