<?php

    /**
     * User: Daschmi (https://daschmi.de)
     * Date: 26.09.2018
     * Time: 13:29
     */
	
	use PHPUnit\Framework\TestCase;
    
    class wpsg_calculationTestTest extends TestCase {
	    	
		public function test1() {
			
			//$this->markTestSkipped(); return;
			
			$b = new \wpsg\wpsg_calculation();
			
			$b->addCountry(1, 0, 0, 7, 19, 0);
			
			$b->addShipping('119', WPSG_BRUTTO, 'c','sss');
			$b->addProduct('119', WPSG_BRUTTO, 'c', 10,'1');
			$b->addPayment('119', WPSG_BRUTTO, 'c', 'ppp');
			
			$ar = $b->getCalculationArray();
			 
			$this->assertTrue($this->compareArray($ar, [
				'sum' => [
					'product_netto' => 1000,
					'product_brutto' => 1190,
					'netto' => 1200,
					'brutto' => 1428,
				]
			]));
			
		}
	
		public function test2() {
			
			//$this->markTestSkipped(); return;
			
			$b = new \wpsg\wpsg_calculation();
			
			$b->addCountry(1, 0, 0, 7, 19, 0);
			
			$b->addProduct('6', WPSG_BRUTTO, 'c', 1,'1');
			$b->addProduct('100', WPSG_BRUTTO, 'b', 1,'2');
			$b->addProduct('30', WPSG_BRUTTO, 'c', 1,'3');
			
			$ar = $b->getCalculationArray();
		
			$this->assertTrue($this->compareArray($ar, [
				'sum' => [
					'product_netto' => 123.71004476557,
					'product_brutto' => 136,
					'netto' => 123.71004476557,
					'brutto' => 136,
				],
				'tax' => [
					'c_1' => [
						'sum' => 5.7478991596639
					] 
					
				]
			]));
		
		}
	
		public function test3() {
			
			//$this->markTestSkipped(); return;
			
			$b = new \wpsg\wpsg_calculation();
			
			$b->addCountry(1, 0, 0, 7, 19, 0);
			
			$b->addProduct('6', WPSG_BRUTTO, 'c', 1,'1');
			$b->addProduct('100', WPSG_BRUTTO, 'b', 1,'2');
			$b->addProduct('30', WPSG_BRUTTO, 'c', 1,'3');
			$b->addVoucher('10',WPSG_BRUTTO,'0',1);
			
			$ar = $b->getCalculationArray();
			 
			$this->assertTrue($this->compareArray($ar, [
				'sum' => [
					'product_netto' => 123.71004476557,
					'product_brutto' => 136, 
					'netto' => 114.613717944572,
					'brutto' => 126,
				],
				'tax' => [
					'c_1' => [
						'sum' => 5.325259516					
					],
					'b_1' => [
						'sum' => 6.06102254					
					]
				]
			]));
			
		}
	
		public function test4() {
		
			//$this->markTestSkipped(); return;
		
			$b = new \wpsg\wpsg_calculation();
		
			$b->addCountry(1, 0, 0, 7, 19, 0, true);
		
			$b->addShipping('119', WPSG_BRUTTO, 'c','sss');
			$b->addProduct('119', WPSG_BRUTTO, 'c', 10,'1');
			$b->addPayment('119', WPSG_BRUTTO, 'c', 'ppp');
			
			$b->setTaxMode(\wpsg\wpsg_calculation::TAXMODE_SMALLBUSINESS);
			
			$ar = $b->getCalculationArray();
			
			$this->assertTrue($this->compareArray($ar, [
				'sum' => [
					'product_netto' => 1000,
					'product_brutto' => 1000,
					'netto' => 1200,
					'brutto' => 1200,
				]
			]));
		
		}
	
		public function test5() {
		
			//$this->markTestSkipped(); return;
		
			$b = new \wpsg\wpsg_calculation();
		
			$b->addCountry(1, 0, 0, 7, 19, 0, true); // Standardland
			$b->addCountry(2, 2, 0, 7, 19, 0, false); // Rechnungsland
		
			$b->addShipping('119', WPSG_BRUTTO, 'c','sss');
			$b->addPayment('119', WPSG_BRUTTO, 'c', 'ppp');
			
			$b->addProduct('119', WPSG_BRUTTO, 'c', 10,'1');
			
			$b->setTaxMode(\wpsg\wpsg_calculation::TAXMODE_B2B);
		
			$ar = $b->getCalculationArray();
		
			$this->assertTrue($this->compareArray($ar, [
				'sum' => [
					'product_netto' => 1000,
					'product_brutto' => 1000,
					'netto' => 1200,
					'brutto' => 1200,
				]
			]));
		
		}
	
		public function test6() {
		 
			$b = new \wpsg\wpsg_calculation();
		
			$b->addCountry(1, 0, 0, 7, 19, 0, true); // Standardland
			$b->addCountry(2, 2, 0, 7, 20, 0, false); // Rechnungsland
		
			$b->addShipping('119', WPSG_BRUTTO, 'c','sss');
			$b->addPayment('119', WPSG_BRUTTO, 'c', 'ppp');
		
			$b->addProduct('120', WPSG_BRUTTO, 'c', 10,'1', false, false,true);
				
			$b->setTaxMode(\wpsg\wpsg_calculation::TAXMODE_B2C);
			
			$ar = $b->getCalculationArray();
		 
			$this->assertTrue($this->compareArray($ar, [
				'sum' => [
					'product_netto' => 1008.4033613445,
					'product_brutto' => 1210.0840336134,
					'netto' => 1208.4033613445,
					'brutto' => 1448.0840336134,
				]
			]));
		
		}
	
		public function test7() {
		 
			$b = new \wpsg\wpsg_calculation();
		
			$b->addCountry(1, 0, 0, 7, 19, 0, true); // Standardland
			$b->addCountry(2, 2, 0, 7, 20, 0, false); // Rechnungsland
		
			$b->addShipping('119', WPSG_BRUTTO, 'c','sss');
			$b->addPayment('119', WPSG_BRUTTO, 'c', 'ppp');
		
			$b->addProduct('120', WPSG_BRUTTO, 'c', 10,'1', false, false,true);
		
			$b->setTaxMode(\wpsg\wpsg_calculation::TAXMODE_B2B);
		
			$ar = $b->getCalculationArray();
			 
			$this->assertTrue($this->compareArray($ar, [
				'sum' => [
					'product_netto' => 1008.4033613445,
					'product_brutto' => 1008.4033613445,
					'netto' => 1208.4033613445,
					'brutto' => 1208.4033613445,
				]
			]));
		
		}
	 	
		private function compareArray($a, $b_check, $key = '') {
			
			if (is_array($b_check)) {
				
				$bReturn = true;
				
				foreach ($b_check as $k => $v) {
										
					$b = $this->compareArray($a[$k], $b_check[$k], $key.'/'.$k);
					if (!$b) $bReturn = false;
					
				}
				
				return $bReturn;
				
			} else {
				
				if (round(wpsg_tf($a), 2, PHP_ROUND_HALF_DOWN) !== round(wpsg_tf($b_check),2, PHP_ROUND_HALF_DOWN)) {
					
					echo "Fehler in ".$key." SOLL: ".$b_check." / IST: ".$a." \r\n";
					
					return false;
					
				}
				else return true;
				
			}
			
		}
    	
    }
